@@ -33,6 +33,7 @@
 #include "glibintl.h"
 
 #include "gioerror.h"
+#include "ginitable.h"
 
 #include "gdbusconnection.h"
 #include "gdbusintrospection.h"
@@ -45,18 +46,32 @@
  * @title: GApplication
  * @short_description: Core application class
  *
- * A #GApplication is the foundation of an application, unique for
- * a given application identifier.  The #GApplication wraps some
- * low-level platform-specific services; it's expected that most
- * software will use a higher-level application class such as
- * #GtkApplication or #MxApplication.
+ * A #GApplication is the foundation of an application, unique for a
+ * given application identifier.  The #GApplication wraps some
+ * low-level platform-specific services and is intended to act as the
+ * foundation for higher-level application classes such as
+ * #GtkApplication or #MxApplication.  In general, you should not use
+ * this class outside of a higher level framework.  By default,
+ * g_application_register_with_data() will invoke g_error() if it is
+ * run in a context where it cannot support its core features.  Note
+ * that g_error() is by default fatal.
  *
- * In addition to single-instance-ness, #GApplication provides support
- * for 'actions', which can be presented to the user in a platform-specific
- * way (e.g. Windows 7 jump lists). Note that these are just simple actions
- * without parameters. For more flexible scriptability, implementing a
- * a separate D-Bus interface is recommended, see e.g.
+ * One of the core features that #GApplication provides is process
+ * uniqueness, in the context of a "session".  The session concept is
+ * platform-dependent, but corresponds roughly to a graphical desktop
+ * login.  When your application is launched again, its arguments
+ * are passed through platform communication to the already running
+ * program.
+ *
+ * In addition, #GApplication provides support for 'actions', which
+ * can be presented to the user in a platform-specific way
+ * (e.g. Windows 7 jump lists). Note that these are just simple
+ * actions without parameters. For more flexible scriptability,
+ * implementing a a separate D-Bus interface is recommended, see e.g.
  * <xref linkend="gdbus-convenience"/>.
+ * 
+ * Finally, #GApplication acts as a basic lifecycle root; see the
+ * g_application_run() and g_application_quit_with_data() methods.
  *
  * Before using #GApplication, you must choose an "application identifier".
  * The expected form of an application identifier is very close to that of
@@ -92,7 +107,7 @@
  *     <void/>
  *     <methodname>InvokeAction</methodname>
  *     <methodparam><modifier>in</modifier><type>s</type><parameter>action</parameter></methodparam>
- *     <methodparam><modifier>in</modifier><type>u</type><parameter>timestamp</parameter></methodparam>
+ *     <methodparam><modifier>in</modifier><type>a{sv}</type><parameter>data</parameter></methodparam>
  *   </methodsynopsis>
  *   <methodsynopsis>
  *     <type>a{s(sb)}</type>
@@ -102,7 +117,7 @@
  *   <methodsynopsis>
  *     <void/>
  *     <methodname>Quit</methodname>
- *     <methodparam><modifier>in</modifier><type>u</type><parameter>timestamp</parameter></methodparam>
+ *     <methodparam><modifier>in</modifier><type>a{sv}</type><parameter>data</parameter></methodparam>
  *   </methodsynopsis>
  *   <methodsynopsis>
  *     <modifier>Signal</modifier>
@@ -115,14 +130,19 @@
  * The <methodname>Activate</methodname> function is called on the existing
  * application instance when a second instance fails to take the bus name.
  * @arguments contains the commandline arguments given to the second instance
- * and @data contains platform-specific additional data, see
- * g_application_format_activation_data().
+ * and @data contains platform-specific additional data.
+ *
+ * On all platforms, @data will have a key "cwd" of type signature
+ * "ay" which contains the working directory of the invoked
+ * executable; this data is defined to be in the default GLib
+ * filesystem encoding for the platform.  See g_filename_to_utf8().
+ *
  * </para>
  * <para>
- * The <methodname>InvokeAction</methodname> function can be called to invoke
- * one of the actions exported by the application. The @timestamp parameter
- * should be taken from the user event that triggered the method call (e.g.
- * a button press event).
+ * The <methodname>InvokeAction</methodname> function can be called to
+ * invoke one of the actions exported by the application.  On X11
+ * platforms, the platform_data argument should have a "timestamp"
+ * parameter of type "u" with the server time of the initiating event.
  * </para>
  * <para>
  * The <methodname>ListActions</methodname> function returns a dictionary
@@ -131,9 +151,11 @@
  * for the action and a boolean that represents if the action is enabled or not.
  * </para>
  * <para>
- * The <methodname>Quit</methodname> function can be called to terminate
- * the application. The @timestamp parameter should be taken from the user
- * event that triggered the method call (e.g. a button press event).
+ * The <methodname>Quit</methodname> function can be called to
+ * terminate the application. The @data parameter contains
+ * platform-specific data.  On X11 platforms, the platform_data
+ * argument should have a "timestamp" parameter of type "u" with the
+ * server time of the initiating event.
  * </para>
  * <para>
  * The <methodname>ActionsChanged</methodname> signal is emitted when the
@@ -146,21 +168,28 @@
  * </refsect2>
  */
 
-G_DEFINE_TYPE (GApplication, g_application, G_TYPE_OBJECT);
+static void initable_iface_init       (GInitableIface      *initable_iface);
+
+G_DEFINE_TYPE_WITH_CODE (GApplication, g_application, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, initable_iface_init));
+
 
 enum
 {
   PROP_0,
 
   PROP_APPLICATION_ID,
+  PROP_REGISTER,
   PROP_DEFAULT_QUIT,
-  PROP_IS_REMOTE
+  PROP_IS_REMOTE,
+  PROP_ARGV,
+  PROP_PLATFORM_DATA
 };
 
 enum
 {
-  QUIT,
-  ACTION,
+  QUIT_WITH_DATA,
+  ACTION_WITH_DATA,
   PREPARE_ACTIVATION,
 
   LAST_SIGNAL
@@ -180,6 +209,10 @@ struct _GApplicationPrivate
   GHashTable *actions; /* name -> GApplicationAction */
   GMainLoop *mainloop;
 
+  GVariant *argv;
+  GVariant *platform_data;
+
+  guint do_register  : 1;
   guint default_quit : 1;
   guint is_remote    : 1;
 
@@ -194,18 +227,31 @@ struct _GApplicationPrivate
 static GApplication *primary_application = NULL;
 static GHashTable *instances_for_appid = NULL;
 
-static void     _g_application_platform_init                    (GApplication  *app); 
-static gboolean _g_application_platform_acquire_single_instance (GApplication  *app,
-                                                                 GError       **error);
+static gboolean initable_init (GInitable     *initable,
+			       GCancellable  *cancellable,
+			       GError       **error);
+
+static gboolean _g_application_platform_init                    (GApplication  *app,
+								 GCancellable  *cancellable,
+								 GError       **error); 
+static gboolean _g_application_platform_register                (GApplication  *app,
+								 gboolean      *unique,
+								 GCancellable  *cancellable,
+								 GError       **error); 
+
 static void     _g_application_platform_remote_invoke_action    (GApplication  *app,
                                                                  const gchar   *action,
-                                                                 guint          timestamp);
+                                                                 GVariant      *platform_data);
 static void     _g_application_platform_remote_quit             (GApplication  *app,
-                                                                 guint          timestamp);
-static void     _g_application_platform_activate                (GApplication  *app,
-                                                                 GVariant      *data) G_GNUC_NORETURN;
+                                                                 GVariant      *platform_data);
 static void     _g_application_platform_on_actions_changed      (GApplication  *app);
 
+static void
+initable_iface_init (GInitableIface *initable_iface)
+{
+  initable_iface->init = initable_init;
+}
+
 #ifdef G_OS_UNIX
 #include "gdbusapplication.c"
 #else
@@ -255,8 +301,8 @@ application_for_appid (const char *appid)
 }
 
 static gboolean
-g_application_default_quit (GApplication *application,
-                            guint         timestamp)
+g_application_default_quit_with_data (GApplication *application,
+				      GVariant     *platform_data)
 {
   g_return_val_if_fail (application->priv->mainloop != NULL, FALSE);
   g_main_loop_quit (application->priv->mainloop);
@@ -273,45 +319,57 @@ g_application_default_run (GApplication *application)
   g_main_loop_run (application->priv->mainloop);
 }
 
-static void
-_g_application_handle_activation (GApplication  *app,
-                                  int            argc,
-                                  char         **argv,
-                                  GVariant      *platform_data)
+static GVariant *
+append_cwd_to_platform_data (GVariant *platform_data)
+{
+  GVariantBuilder builder;
+  gchar *cwd;
+  GVariant *result;
+
+  cwd = g_get_current_dir ();
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+  if (cwd)
+    g_variant_builder_add (&builder, "{sv}",
+			   "cwd",
+			   g_variant_new_byte_array (cwd, -1));
+  g_free (cwd);
+
+  if (platform_data)
+    {
+      GVariantIter iter;
+      GVariant *item;
+
+      g_variant_iter_init (&iter, platform_data);
+      while (g_variant_iter_next (&iter, "@{sv}", &item))
+	{
+	  g_variant_builder_add_value (&builder, item);
+	  g_variant_unref (item);
+	}
+    }
+  result = g_variant_builder_end (&builder);
+  return result;
+}
+
+static GVariant *
+variant_from_argv (int    argc,
+		   char **argv)
 {
   GVariantBuilder builder;
-  GVariant *message;
   int i;
 
-  g_variant_builder_init (&builder, G_VARIANT_TYPE ("(aaya{sv})"));
-  g_variant_builder_open (&builder, G_VARIANT_TYPE ("aay"));
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("aay"));
 
   for (i = 1; i < argc; i++)
     {
-      int j;
       guint8 *argv_bytes;
 
-      g_variant_builder_open (&builder, G_VARIANT_TYPE ("ay"));
-
       argv_bytes = (guint8*) argv[i];
-      for (j = 0; argv_bytes[j]; j++)
-        g_variant_builder_add_value (&builder,
-                                     g_variant_new_byte (argv_bytes[j]));
-      g_variant_builder_close (&builder);
+      g_variant_builder_add_value (&builder,
+				   g_variant_new_byte_array (argv_bytes, -1));
     }
-  g_variant_builder_close (&builder);
-
-  if (platform_data)
-    g_variant_builder_add (&builder, "@a{sv}", platform_data);
-  else
-    {
-      g_variant_builder_open (&builder, G_VARIANT_TYPE ("a{sv}"));
-      g_variant_builder_close (&builder);
-    }
-
-  message = g_variant_builder_end (&builder);
-  _g_application_platform_activate (app, message);
-  g_variant_unref (message);
+  
+  return g_variant_builder_end (&builder);
 }
 
 static gboolean
@@ -335,6 +393,24 @@ queue_actions_change_notification (GApplication *application)
     priv->actions_changed_id = g_timeout_add (0, timeout_handle_actions_changed, application);
 }
 
+static gboolean
+initable_init (GInitable     *initable,
+               GCancellable  *cancellable,
+               GError       **error)
+{
+  GApplication *app = G_APPLICATION (initable);
+  gboolean unique;
+
+  if (!_g_application_platform_init (app, cancellable, error))
+    return FALSE;
+
+  if (app->priv->do_register &&
+      !_g_application_platform_register (app, &unique, cancellable ,error))
+    return FALSE;
+
+  return TRUE;
+}
+
 static void
 g_application_action_free (gpointer data)
 {
@@ -349,15 +425,36 @@ g_application_action_free (gpointer data)
     }
 }
 
-
 /**
  * g_application_new:
  * @appid: System-dependent application identifier
+ * @argc: Number of arguments in @argv
+ * @argv: (allow-none) (array length=argc): Argument vector, usually from the <parameter>argv</parameter> parameter of main() 
  *
- * Create a new #GApplication.  The application is initially in
- * "remote" mode.  Almost certainly, you want to call
- * g_application_register() immediately after this function, which
- * will finish initialization.
+ * Create a new #GApplication.  This uses a platform-specific
+ * mechanism to ensure the current process is the unique owner of the
+ * application (as defined by the @appid). If successful, the
+ * #GApplication:is-remote property will be %FALSE, and it is safe to
+ * continue creating other resources such as graphics windows.
+ *
+ * If the given @appid is already running in another process, the the
+ * GApplication::activate_with_data signal will be emitted in the
+ * remote process, with the data from @argv and other
+ * platform-specific data available.  Subsequently the
+ * #GApplication:default-quit property will be evaluated.  If it's
+ * %TRUE, then the current process will terminate.  If %FALSE, then
+ * the application remains in the #GApplication:is-remote state, and
+ * you can e.g. call g_application_invoke_action(). Note that proxy
+ * instances should not call g_application_add_action().
+ *
+ * This function may do synchronous I/O to obtain unique ownership
+ * of the application id, and will block the calling thread in this
+ * case.
+ *
+ * If the environment does not support the basic functionality of
+ * #GApplication, this function will invoke g_error(), which by
+ * default is a fatal operation.  This may arise for example on
+ * UNIX systems using D-Bus when the session bus is not available.
  *
  * As a convenience, this function is defined to call g_type_init() as
  * its very first action.
@@ -367,79 +464,133 @@ g_application_action_free (gpointer data)
  * Since: 2.26
  */
 GApplication *
-g_application_new (const gchar *appid)
+g_application_new (const gchar *appid,
+		   int          argc,
+		   char       **argv)
 {
+  GObject *app;
+  GError *error = NULL;
+  GVariant *argv_variant;
+
   g_type_init ();
 
-  return G_APPLICATION (g_object_new (G_TYPE_APPLICATION, "application-id", appid, NULL));
-}
-
-/**
- * g_application_register_with_data:
- * @application: A #GApplication
- * @argc: System argument count
- * @argv: (array length=argc): System argument vector
- * @platform_data: (allow-none): Arbitrary platform-specific data, must have signature "a{sv}"
- *
- * Ensure the current process is the unique owner of the application.
- * If successful, the #GApplication:is-remote property will be changed
- * to %FALSE, and it is safe to continue creating other resources
- * such as graphics windows.
- *
- * If the given @appid is already running in another process, the
- * #GApplication:default-exit property will be evaluated.  If it's
- * %TRUE, then a platform-specific action such as bringing any
- * graphics windows associated with the application to the foreground
- * may be initiated.  After that, the current process will terminate.
- * If %FALSE, then the application remains in the #GApplication:is-remote
- * state, and you can e.g. call g_application_invoke_action().
- *
- * This function may do synchronous I/O to obtain unique ownership
- * of the application id, and will block the calling thread in this
- * case.
- */
-void
-g_application_register_with_data (GApplication  *application,
-                                  gint           argc,
-                                  gchar        **argv,
-                                  GVariant      *platform_data)
-{
-  g_return_if_fail (application->priv->appid != NULL);
-  g_return_if_fail (application->priv->is_remote);
-  g_return_if_fail (platform_data == NULL
-                    || strcmp (g_variant_get_type_string (platform_data), "a{sv}") == 0);
-
-  if (!_g_application_platform_acquire_single_instance (application, NULL))
+  g_return_val_if_fail (appid != NULL, NULL);
+  
+  argv_variant = variant_from_argv (argc, argv);
+  
+  app = g_initable_new (G_TYPE_APPLICATION, 
+			NULL,
+			&error,
+			"application-id", appid, 
+			"argv", argv_variant, 
+			NULL);
+  if (!app)
     {
-      if (application->priv->default_quit)
-        _g_application_handle_activation (application, argc, argv, platform_data);
-      else
-        return;
+      g_error ("%s", error->message);
+      g_clear_error (&error);
+      return NULL;
     }
-
-  application->priv->is_remote = FALSE;
-
-  _g_application_platform_init (application);
+  return G_APPLICATION (app);
 }
 
 /**
- * g_application_new_and_register:
- * @appid: An application identifier
- * @argc: System argument count
- * @argv: (array length=argc): System argument vector
+ * g_application_try_new:
+ * @appid: System-dependent application identifier
+ * @argc: Number of arguments in @argv
+ * @argv: (allow-none) (array length=argc): Argument vector, usually from the <parameter>argv</parameter> parameter of main() 
+ * @error: a #GError
  *
- * This is a convenience function which combines g_application_new()
- * with g_application_register_with_data(). Therefore, it may block
- * the calling thread just like g_application_register_with_data().
+ * This function is similar to g_application_new(), but allows for
+ * more graceful fallback if the environment doesn't support the
+ * basic #GApplication functionality.
+ *
+ * Returns: (transfer full): An application instance
+ *
+ * Since: 2.26
  */
 GApplication *
-g_application_new_and_register (const gchar  *appid,
-                                gint          argc,
-                                gchar       **argv)
+g_application_try_new (const gchar *appid,
+		       int          argc,
+		       char       **argv,
+		       GError     **error)
 {
-  GApplication *app = g_application_new (appid);
-  g_application_register_with_data (app, argc, argv, NULL);
-  return app;
+  GVariant *argv_variant;
+
+  g_type_init ();
+
+  g_return_val_if_fail (appid != NULL, NULL);
+  
+  argv_variant = variant_from_argv (argc, argv);
+  
+  return G_APPLICATION (g_initable_new (G_TYPE_APPLICATION, 
+					NULL,
+					error,
+					"application-id", appid, 
+					"argv", argv_variant, 
+					NULL));
+}
+
+/**
+ * g_application_unregistered_try_new:
+ * @appid: System-dependent application identifier
+ * @argc: Number of arguments in @argv
+ * @argv: (allow-none) (array length=argc): Argument vector, usually from the <parameter>argv</parameter> parameter of main() 
+ * @error: a #GError
+ *
+ * This function is similar to g_application_try_new(), but also
+ * sets the GApplication:register property to %FALSE.  You can later
+ * call g_application_register() to complete initialization.
+ *
+ * Returns: (transfer full): An application instance
+ *
+ * Since: 2.26
+ */
+GApplication *
+g_application_unregistered_try_new (const gchar *appid,
+				    int          argc,
+				    char       **argv,
+				    GError     **error)
+{
+  GVariant *argv_variant;
+
+  g_type_init ();
+
+  g_return_val_if_fail (appid != NULL, NULL);
+  
+  argv_variant = variant_from_argv (argc, argv);
+  
+  return G_APPLICATION (g_initable_new (G_TYPE_APPLICATION, 
+					NULL,
+					error,
+					"application-id", appid, 
+					"argv", argv_variant, 
+					"register", FALSE,
+					NULL));
+}
+
+/**
+ * g_application_register:
+ * @application: a #GApplication
+ *
+ * By default, #GApplication ensures process uniqueness when
+ * initialized, but this behavior is controlled by the
+ * GApplication:register property.  If it was given as %FALSE at
+ * construction time, this function allows you to later attempt
+ * to ensure uniqueness.
+ *
+ * Returns: %TRUE if registration was successful
+ */
+gboolean
+g_application_register (GApplication *application)
+{
+  gboolean unique;
+
+  g_return_val_if_fail (G_IS_APPLICATION (application), FALSE);
+  g_return_val_if_fail (application->priv->is_remote, FALSE);
+
+  if (!_g_application_platform_register (application, &unique, NULL, NULL))
+    return FALSE;
+  return unique;
 }
 
 /**
@@ -520,8 +671,7 @@ g_application_remove_action (GApplication *application,
  * g_application_invoke_action:
  * @application: a #GApplication
  * @name: the name of the action to invoke
- * @timestamp: the timestamp that is going to be passed to
- *   the #GApplication::action signal
+ * @platform_data: (allow-none): platform-specific event data
  *
  * Invokes the action @name of the passed #GApplication.
  *
@@ -541,31 +691,41 @@ g_application_remove_action (GApplication *application,
 void
 g_application_invoke_action (GApplication *application,
                              const gchar  *name,
-                             guint         timestamp)
+                             GVariant     *platform_data)
 {
   GApplicationPrivate *priv;
   GApplicationAction *action;
 
   g_return_if_fail (G_IS_APPLICATION (application));
   g_return_if_fail (name != NULL);
+  g_return_if_fail (platform_data == NULL
+                    || g_variant_is_of_type (platform_data, G_VARIANT_TYPE ("a{sv}")));
+  
+  if (platform_data == NULL)
+    platform_data = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
+  else
+    g_variant_ref (platform_data);
 
   priv = application->priv;
-
+  
   if (priv->is_remote)
     {
-      _g_application_platform_remote_invoke_action (application, name, timestamp);
-      return;
+      _g_application_platform_remote_invoke_action (application, name, platform_data);
+      goto out;
     }
 
   action = g_hash_table_lookup (priv->actions, name);
   g_return_if_fail (action != NULL);
   if (!action->enabled)
-    return;
+    goto out;
 
-  g_signal_emit (application, application_signals[ACTION],
+  g_signal_emit (application, application_signals[ACTION_WITH_DATA],
                  g_quark_from_string (name),
                  name,
-                 timestamp);
+                 platform_data);
+
+ out:
+  g_variant_unref (platform_data);
 }
 
 /**
@@ -737,9 +897,9 @@ g_application_run (GApplication *application)
 }
 
 /**
- * g_application_quit:
+ * g_application_quit_with_data:
  * @application: a #GApplication
- * @timestamp: Platform-specific event timestamp, may be 0 for default
+ * @platform_data: (allow-none): platform-specific data
  *
  * Request that the application quits.
  *
@@ -756,20 +916,29 @@ g_application_run (GApplication *application)
  * Since: 2.26
  */
 gboolean
-g_application_quit (GApplication *application,
-                    guint         timestamp)
+g_application_quit_with_data (GApplication *application,
+			      GVariant     *platform_data)
 {
   gboolean retval = FALSE;
 
   g_return_val_if_fail (G_IS_APPLICATION (application), FALSE);
+  g_return_val_if_fail (platform_data == NULL
+			|| g_variant_is_of_type (platform_data, G_VARIANT_TYPE ("a{sv}")), FALSE);
+
+  if (platform_data == NULL)
+    platform_data = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
+  else
+    g_variant_ref (platform_data);
 
   if (application->priv->is_remote)
     {
-       _g_application_platform_remote_quit (application, timestamp);
+       _g_application_platform_remote_quit (application, platform_data);
        retval = TRUE;
     }
   else
-    g_signal_emit (application, application_signals[QUIT], 0, timestamp, &retval);
+    g_signal_emit (application, application_signals[QUIT_WITH_DATA], 0, platform_data, &retval);
+
+  g_variant_unref (platform_data);
 
   return retval;
 }
@@ -816,6 +985,8 @@ g_application_get_id (GApplication *application)
  * g_application_is_remote:
  * @application: a #GApplication
  *
+ * Returns whether the object represents a proxy for a remote application.
+ *
  * Returns: %TRUE if this object represents a proxy for a remote application.
  */
 gboolean
@@ -837,7 +1008,9 @@ g_application_init (GApplication *app)
                                               NULL,
                                               g_application_action_free);
   app->priv->default_quit = TRUE;
+  app->priv->do_register = TRUE;
   app->priv->is_remote = TRUE;
+  app->priv->platform_data = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
 }
 
 static void
@@ -862,6 +1035,18 @@ g_application_get_property (GObject    *object,
       g_value_set_boolean (value, g_application_is_remote (app));
       break;
 
+    case PROP_REGISTER:
+      g_value_set_boolean (value, app->priv->do_register);
+      break;
+
+    case PROP_ARGV:
+      g_value_set_variant (value, app->priv->argv);
+      break;
+
+    case PROP_PLATFORM_DATA:
+      g_value_set_variant (value, app->priv->platform_data);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -886,6 +1071,23 @@ g_application_set_property (GObject      *object,
       app->priv->default_quit = g_value_get_boolean (value);
       break;
 
+    case PROP_REGISTER:
+      app->priv->do_register = g_value_get_boolean (value);
+      break;
+
+    case PROP_ARGV:
+      app->priv->argv = g_value_dup_variant (value);
+      break;
+
+    case PROP_PLATFORM_DATA:
+      {
+	GVariant *platform_data = g_value_get_variant (value);
+	if (app->priv->platform_data)
+	  g_variant_unref (app->priv->platform_data);
+	app->priv->platform_data = g_variant_ref_sink (append_cwd_to_platform_data (platform_data));
+      }
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -962,12 +1164,12 @@ g_application_class_init (GApplicationClass *klass)
   gobject_class->finalize = g_application_finalize;
 
   klass->run = g_application_default_run;
-  klass->quit = g_application_default_quit;
+  klass->quit_with_data = g_application_default_quit_with_data;
 
   /**
-   * GApplication::quit:
+   * GApplication::quit-with-data:
    * @application: the object on which the signal is emitted
-   * @timestamp: Platform-specific event timestamp, may be 0 for default
+   * @platform_data: Platform-specific data, or %NULL
    *
    * This signal is emitted when the Quit action is invoked on the
    * application.
@@ -978,21 +1180,21 @@ g_application_class_init (GApplicationClass *klass)
    * Returns: %TRUE if the signal has been handled, %FALSE to continue
    *   signal emission
    */
-  application_signals[QUIT] =
-    g_signal_new (g_intern_static_string ("quit"),
+  application_signals[QUIT_WITH_DATA] =
+    g_signal_new (g_intern_static_string ("quit-with-data"),
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (GApplicationClass, quit),
+                  G_STRUCT_OFFSET (GApplicationClass, quit_with_data),
                   g_signal_accumulator_true_handled, NULL,
-                  _gio_marshal_BOOLEAN__UINT,
+                  _gio_marshal_BOOLEAN__VARIANT,
                   G_TYPE_BOOLEAN, 1,
-                  G_TYPE_UINT);
+                  G_TYPE_VARIANT);
 
   /**
-   * GApplication::action:
+   * GApplication::action-with-data:
    * @application: the object on which the signal is emitted
    * @name: The name of the activated action
-   * @timestamp: Platform-specific event timestamp, may be 0 for default
+   * @platform_data: Platform-specific data, or %NULL
    *
    * This signal is emitted when an action is activated. The action name
    * is passed as the first argument, but also as signal detail, so it
@@ -1000,22 +1202,22 @@ g_application_class_init (GApplicationClass *klass)
    *
    * The signal is never emitted for disabled actions.
    */
-  application_signals[ACTION] =
-    g_signal_new (g_intern_static_string ("action"),
+  application_signals[ACTION_WITH_DATA] =
+    g_signal_new (g_intern_static_string ("action-with-data"),
                   G_OBJECT_CLASS_TYPE (klass),
                   G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED,
-                  G_STRUCT_OFFSET (GApplicationClass, action),
+                  G_STRUCT_OFFSET (GApplicationClass, action_with_data),
                   NULL, NULL,
-                  _gio_marshal_VOID__STRING_UINT,
+                  _gio_marshal_VOID__STRING_VARIANT,
                   G_TYPE_NONE, 2,
                   G_TYPE_STRING,
-                  G_TYPE_UINT);
+                  G_TYPE_VARIANT);
 
    /**
    * GApplication::prepare-activation:
    * @application: the object on which the signal is emitted
    * @arguments: A #GVariant with the signature "aay"
-   * @platform_data: A #GVariant with the signature "a{sv}"
+   * @platform_data: A #GVariant with the signature "a{sv}", or %NULL
    *
    * This signal is emitted when a non-primary process for a given
    * application is invoked while your application is running; for
@@ -1030,12 +1232,12 @@ g_application_class_init (GApplicationClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GApplicationClass, prepare_activation),
                   NULL, NULL,
-                  _gio_marshal_VOID__BOXED_BOXED,
+                  _gio_marshal_VOID__VARIANT_VARIANT,
                   G_TYPE_NONE, 2,
                   G_TYPE_VARIANT,
                   G_TYPE_VARIANT);
 
-   /**
+  /**
    * GApplication:application-id:
    *
    * The unique identifier for this application.  See the documentation for
@@ -1052,6 +1254,42 @@ g_application_class_init (GApplicationClass *klass)
                                                         G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GApplication:argv:
+   *
+   * The argument vector given to this application.  It must be a #GVariant
+   * with a type signature "aay".
+   *
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_ARGV,
+                                   g_param_spec_variant ("argv",
+						        P_("Argument vector"),
+						        P_("System argument vector with type signature aay"),
+						        G_VARIANT_TYPE ("aay"),
+                                                        NULL,
+						        G_PARAM_READWRITE |
+						        G_PARAM_CONSTRUCT_ONLY |
+						        G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GApplication:platform-data:
+   *
+   * Platform-specific data retrieved from the operating system
+   * environment.  It must be a #GVariant with type signature "a{sv}".
+   *
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_PLATFORM_DATA,
+                                   g_param_spec_variant ("platform-data",
+						         P_("Platform data"),
+						         P_("Environmental data, must have type signature a{sv}"),
+						         G_VARIANT_TYPE ("a{sv}"),
+                                                         NULL,
+						         G_PARAM_READWRITE |
+						         G_PARAM_CONSTRUCT_ONLY |
+						         G_PARAM_STATIC_STRINGS));
+
   /**
    * GApplication:default-quit:
    *
@@ -1085,6 +1323,23 @@ g_application_class_init (GApplicationClass *klass)
                                                          P_("Whether this application is a proxy for another process"),
                                                          TRUE,
                                                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GApplication:register:
+   *
+   * If this property is %FALSE, the application construction will not attempt
+   * to ensure process uniqueness, and the application is guaranteed to be in the
+   * remote state.  See GApplication:is-remote.
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_REGISTER,
+                                   g_param_spec_boolean ("register",
+                                                         P_("Register"),
+                                                         P_("If false, do not "),
+                                                         TRUE,
+                                                         G_PARAM_READWRITE | 
+							 G_PARAM_CONSTRUCT_ONLY |
+							 G_PARAM_STATIC_STRINGS));
 }
 
 #define __G_APPLICATION_C__
