@@ -46,7 +46,6 @@ typedef struct
   GPollFD pollfd;
   GCancellable *cancellable;
   gulong cancelled_tag;
-  GObject *object;
 } FDSource;
 
 static gboolean 
@@ -59,7 +58,7 @@ fd_source_prepare (GSource *source,
   return g_cancellable_is_cancelled (fd_source->cancellable);
 }
 
-static gboolean
+static gboolean 
 fd_source_check (GSource *source)
 {
   FDSource *fd_source = (FDSource *)source;
@@ -76,18 +75,14 @@ fd_source_dispatch (GSource     *source,
 
 {
   GFDSourceFunc func = (GFDSourceFunc)callback;
-  GFDSourceObjectFunc func2 = (GFDSourceObjectFunc)callback;
   FDSource *fd_source = (FDSource *)source;
 
   g_warn_if_fail (func != NULL);
 
-  if (fd_source->object)
-    return (*func2) (fd_source->object, fd_source->pollfd.revents, user_data);
-  else
-    return (*func) (user_data, fd_source->pollfd.revents, fd_source->pollfd.fd);
+  return (*func) (user_data, fd_source->pollfd.revents, fd_source->pollfd.fd);
 }
 
-static void
+static void 
 fd_source_finalize (GSource *source)
 {
   FDSource *fd_source = (FDSource *)source;
@@ -98,9 +93,6 @@ fd_source_finalize (GSource *source)
 
   if (fd_source->cancellable)
     g_object_unref (fd_source->cancellable);
-
-  if (fd_source->object)
-    g_object_unref (fd_source->object);
 }
 
 static GSourceFuncs fd_source_funcs = {
@@ -120,10 +112,9 @@ fd_source_cancelled_cb (GCancellable *cancellable,
 }
 
 GSource *
-_g_fd_source_new_with_object (GObject      *object,
-			      int           fd,
-			      gushort       events,
-			      GCancellable *cancellable)
+_g_fd_source_new (int           fd,
+		  gushort       events,
+		  GCancellable *cancellable)
 {
   GSource *source;
   FDSource *fd_source;
@@ -133,27 +124,16 @@ _g_fd_source_new_with_object (GObject      *object,
 
   if (cancellable)
     fd_source->cancellable = g_object_ref (cancellable);
-
-  if (object)
-    fd_source->object = g_object_ref (object);
-
+  
   fd_source->pollfd.fd = fd;
   fd_source->pollfd.events = events;
   g_source_add_poll (source, &fd_source->pollfd);
 
   if (cancellable)
     fd_source->cancelled_tag =
-      g_cancellable_connect (cancellable,
+      g_cancellable_connect (cancellable, 
 			     (GCallback)fd_source_cancelled_cb,
 			     NULL, NULL);
-
+  
   return source;
 }
-
-GSource *
-_g_fd_source_new (int           fd,
-		  gushort       events,
-		  GCancellable *cancellable)
-{
-  return _g_fd_source_new_with_object (NULL, fd, events, cancellable);
-}
