@@ -25,10 +25,16 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "gioerror.h"
 #include "gdbusutils.h"
 #include "gdbusaddress.h"
 #include "gdbuserror.h"
 #include "gioenumtypes.h"
+#include "gnetworkaddress.h"
+#include "gsocketclient.h"
+#include "giostream.h"
+#include "gasyncresult.h"
+#include "gsimpleasyncresult.h"
 #include "gdbusprivate.h"
 
 #ifdef G_OS_UNIX
@@ -156,7 +162,7 @@ is_valid_unix (const gchar  *address_entry,
       g_set_error (error,
                    G_IO_ERROR,
                    G_IO_ERROR_INVALID_ARGUMENT,
-                   _("Address `%s' is invalid (need exactly one of path, tmpdir or abstract keys"),
+                   _("Address `%s' is invalid (need exactly one of path, tmpdir or abstract keys)"),
                    address_entry);
       goto out;
     }
