@@ -63,119 +63,74 @@
  * #GDrive in that API.
  **/
 
-static void g_drive_base_init (gpointer g_class);
-static void g_drive_class_init (gpointer g_class,
-				 gpointer class_data);
-
-GType
-g_drive_get_type (void)
-{
-  static volatile gsize g_define_type_id__volatile = 0;
-
-  if (g_once_init_enter (&g_define_type_id__volatile))
-    {
-      const GTypeInfo drive_info =
-      {
-        sizeof (GDriveIface), /* class_size */
-	g_drive_base_init,   /* base_init */
-	NULL,		/* base_finalize */
-	g_drive_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	0,
-	0,              /* n_preallocs */
-	NULL
-      };
-      GType g_define_type_id =
-	g_type_register_static (G_TYPE_INTERFACE, I_("GDrive"),
-				&drive_info, 0);
-
-      g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_OBJECT);
-
-      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
-    }
-
-  return g_define_type_id__volatile;
-}
+typedef GDriveIface GDriveInterface;
+G_DEFINE_INTERFACE(GDrive, g_drive, G_TYPE_OBJECT)
 
 static void
-g_drive_class_init (gpointer g_class,
-                    gpointer class_data)
+g_drive_default_init (GDriveInterface *iface)
 {
-}
+  /**
+   * GDrive::changed:
+   * @drive: a #GDrive.
+   *
+   * Emitted when the drive's state has changed.
+   **/
+  g_signal_new (I_("changed"),
+		G_TYPE_DRIVE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GDriveIface, changed),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 
-static void
-g_drive_base_init (gpointer g_class)
-{
-  static gboolean initialized = FALSE;
+  /**
+   * GDrive::disconnected:
+   * @drive: a #GDrive.
+   *
+   * This signal is emitted when the #GDrive have been
+   * disconnected. If the recipient is holding references to the
+   * object they should release them so the object can be
+   * finalized.
+   **/
+  g_signal_new (I_("disconnected"),
+		G_TYPE_DRIVE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GDriveIface, disconnected),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 
-  if (! initialized)
-    {
-      /**
-      * GDrive::changed:
-      * @drive: a #GDrive.
-      * 
-      * Emitted when the drive's state has changed.
-      **/
-      g_signal_new (I_("changed"),
-                    G_TYPE_DRIVE,
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (GDriveIface, changed),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
+  /**
+   * GDrive::eject-button:
+   * @drive: a #GDrive.
+   *
+   * Emitted when the physical eject button (if any) of a drive has
+   * been pressed.
+   **/
+  g_signal_new (I_("eject-button"),
+		G_TYPE_DRIVE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GDriveIface, eject_button),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 
-      /**
-      * GDrive::disconnected:
-      * @drive: a #GDrive.
-      * 
-      * This signal is emitted when the #GDrive have been
-      * disconnected. If the recipient is holding references to the
-      * object they should release them so the object can be
-      * finalized.
-      **/
-      g_signal_new (I_("disconnected"),
-                    G_TYPE_DRIVE,
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (GDriveIface, disconnected),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
-
-      /**
-      * GDrive::eject-button:
-      * @drive: a #GDrive.
-      * 
-      * Emitted when the physical eject button (if any) of a drive has
-      * been pressed.
-      **/
-      g_signal_new (I_("eject-button"),
-                    G_TYPE_DRIVE,
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (GDriveIface, eject_button),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
-
-      /**
-      * GDrive::stop-button:
-      * @drive: a #GDrive.
-      *
-      * Emitted when the physical stop button (if any) of a drive has
-      * been pressed.
-      *
-      * Since: 2.22
-      **/
-      g_signal_new (I_("stop-button"),
-                    G_TYPE_DRIVE,
-                    G_SIGNAL_RUN_LAST,
-                    G_STRUCT_OFFSET (GDriveIface, stop_button),
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
-                    G_TYPE_NONE, 0);
-
-      initialized = TRUE;
-    }
+  /**
+   * GDrive::stop-button:
+   * @drive: a #GDrive.
+   *
+   * Emitted when the physical stop button (if any) of a drive has
+   * been pressed.
+   *
+   * Since: 2.22
+   **/
+  g_signal_new (I_("stop-button"),
+		G_TYPE_DRIVE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GDriveIface, stop_button),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 }
 
 /**
@@ -249,7 +204,7 @@ g_drive_has_volumes (GDrive *drive)
  * The returned list should be freed with g_list_free(), after
  * its elements have been unreffed with g_object_unref().
  * 
- * Returns: #GList containing any #GVolume objects on the given @drive.
+ * Returns: (element-type GVolume) (transfer full): #GList containing any #GVolume objects on the given @drive.
  **/
 GList *
 g_drive_get_volumes (GDrive *drive)
@@ -386,6 +341,8 @@ g_drive_can_poll_for_media (GDrive *drive)
  * When the operation is finished, @callback will be called.
  * You can then call g_drive_eject_finish() to obtain the
  * result of the operation.
+ *
+ * Deprecated: 2.22: Use g_drive_eject_with_operation() instead.
  **/
 void
 g_drive_eject (GDrive              *drive,
@@ -422,6 +379,8 @@ g_drive_eject (GDrive              *drive,
  * 
  * Returns: %TRUE if the drive has been ejected successfully,
  *     %FALSE otherwise.
+ *
+ * Deprecated: 2.22: Use g_drive_eject_with_operation_finish() instead.
  **/
 gboolean
 g_drive_eject_finish (GDrive        *drive,
@@ -445,6 +404,91 @@ g_drive_eject_finish (GDrive        *drive,
   return (* iface->eject_finish) (drive, result, error);
 }
 
+/**
+ * g_drive_eject_with_operation:
+ * @drive: a #GDrive.
+ * @flags: flags affecting the unmount if required for eject
+ * @mount_operation: a #GMountOperation or %NULL to avoid user interaction.
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
+ * @callback: a #GAsyncReadyCallback, or %NULL.
+ * @user_data: user data passed to @callback.
+ *
+ * Ejects a drive. This is an asynchronous operation, and is
+ * finished by calling g_drive_eject_with_operation_finish() with the @drive
+ * and #GAsyncResult data returned in the @callback.
+ *
+ * Since: 2.22
+ **/
+void
+g_drive_eject_with_operation (GDrive              *drive,
+                              GMountUnmountFlags   flags,
+                              GMountOperation     *mount_operation,
+                              GCancellable        *cancellable,
+                              GAsyncReadyCallback  callback,
+                              gpointer             user_data)
+{
+  GDriveIface *iface;
+
+  g_return_if_fail (G_IS_DRIVE (drive));
+
+  iface = G_DRIVE_GET_IFACE (drive);
+
+  if (iface->eject == NULL && iface->eject_with_operation == NULL)
+    {
+      g_simple_async_report_error_in_idle (G_OBJECT (drive),
+					   callback, user_data,
+					   G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+					   /* Translators: This is an error
+					    * message for drive objects that
+					    * don't implement any of eject or eject_with_operation. */
+					   _("drive doesn't implement eject or eject_with_operation"));
+      return;
+    }
+
+  if (iface->eject_with_operation != NULL)
+    (* iface->eject_with_operation) (drive, flags, mount_operation, cancellable, callback, user_data);
+  else
+    (* iface->eject) (drive, flags, cancellable, callback, user_data);
+}
+
+/**
+ * g_drive_eject_with_operation_finish:
+ * @drive: a #GDrive.
+ * @result: a #GAsyncResult.
+ * @error: a #GError location to store the error occuring, or %NULL to
+ *     ignore.
+ *
+ * Finishes ejecting a drive. If any errors occurred during the operation,
+ * @error will be set to contain the errors and %FALSE will be returned.
+ *
+ * Returns: %TRUE if the drive was successfully ejected. %FALSE otherwise.
+ *
+ * Since: 2.22
+ **/
+gboolean
+g_drive_eject_with_operation_finish (GDrive        *drive,
+                                     GAsyncResult  *result,
+                                     GError       **error)
+{
+  GDriveIface *iface;
+
+  g_return_val_if_fail (G_IS_DRIVE (drive), FALSE);
+  g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+        return FALSE;
+    }
+
+  iface = G_DRIVE_GET_IFACE (drive);
+  if (iface->eject_with_operation_finish != NULL)
+    return (* iface->eject_with_operation_finish) (drive, result, error);
+  else
+    return (* iface->eject_finish) (drive, result, error);
+}
+
 /**
  * g_drive_poll_for_media:
  * @drive: a #GDrive.
@@ -619,11 +663,36 @@ g_drive_can_start (GDrive *drive)
   return (* iface->can_start) (drive);
 }
 
+/**
+ * g_drive_can_start_degraded:
+ * @drive: a #GDrive.
+ *
+ * Checks if a drive can be started degraded.
+ *
+ * Returns: %TRUE if the @drive can be started degraded, %FALSE otherwise.
+ *
+ * Since: 2.22
+ */
+gboolean
+g_drive_can_start_degraded (GDrive *drive)
+{
+  GDriveIface *iface;
+
+  g_return_val_if_fail (G_IS_DRIVE (drive), FALSE);
+
+  iface = G_DRIVE_GET_IFACE (drive);
+
+  if (iface->can_start_degraded == NULL)
+    return FALSE;
+
+  return (* iface->can_start_degraded) (drive);
+}
+
 /**
  * g_drive_start:
  * @drive: a #GDrive.
  * @flags: flags affecting the start operation.
- * @start_operation: a #GMountOperation or %NULL to avoid user interaction.
+ * @mount_operation: a #GMountOperation or %NULL to avoid user interaction.
  * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback, or %NULL.
  * @user_data: user data to pass to @callback
@@ -639,7 +708,7 @@ g_drive_can_start (GDrive *drive)
 void
 g_drive_start (GDrive              *drive,
                GDriveStartFlags     flags,
-               GMountOperation     *start_operation,
+               GMountOperation     *mount_operation,
                GCancellable        *cancellable,
                GAsyncReadyCallback  callback,
                gpointer             user_data)
@@ -658,7 +727,7 @@ g_drive_start (GDrive              *drive,
       return;
     }
 
-  (* iface->start) (drive, flags, start_operation, cancellable, callback, user_data);
+  (* iface->start) (drive, flags, mount_operation, cancellable, callback, user_data);
 }
 
 /**
@@ -725,6 +794,7 @@ g_drive_can_stop (GDrive *drive)
  * g_drive_stop:
  * @drive: a #GDrive.
  * @flags: flags affecting the unmount if required for stopping.
+ * @mount_operation: a #GMountOperation or %NULL to avoid user interaction.
  * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback, or %NULL.
  * @user_data: user data to pass to @callback
@@ -740,6 +810,7 @@ g_drive_can_stop (GDrive *drive)
 void
 g_drive_stop (GDrive               *drive,
               GMountUnmountFlags    flags,
+              GMountOperation      *mount_operation,
               GCancellable         *cancellable,
               GAsyncReadyCallback   callback,
               gpointer              user_data)
@@ -758,7 +829,7 @@ g_drive_stop (GDrive               *drive,
       return;
     }
 
-  (* iface->stop) (drive, flags, cancellable, callback, user_data);
+  (* iface->stop) (drive, flags, mount_operation, cancellable, callback, user_data);
 }
 
 /**
