@@ -748,6 +748,7 @@ g_bus_type_get_type (void)
     {
       static const GEnumValue values[] = {
         { G_BUS_TYPE_STARTER, "G_BUS_TYPE_STARTER", "starter" },
+        { G_BUS_TYPE_NONE, "G_BUS_TYPE_NONE", "none" },
         { G_BUS_TYPE_SYSTEM, "G_BUS_TYPE_SYSTEM", "system" },
         { G_BUS_TYPE_SESSION, "G_BUS_TYPE_SESSION", "session" },
         { 0, NULL, NULL }
@@ -812,6 +813,7 @@ g_dbus_proxy_flags_get_type (void)
         { G_DBUS_PROXY_FLAGS_NONE, "G_DBUS_PROXY_FLAGS_NONE", "none" },
         { G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES, "G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES", "do-not-load-properties" },
         { G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS, "G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS", "do-not-connect-signals" },
+        { G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START, "G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START", "do-not-auto-start" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
