@@ -29,8 +29,31 @@
 
 #include "galias.h"
 
+/**
+ * SECTION: checksum
+ * @title: Data Checksums
+ * @short_description: Computes the checksum for data
+ *
+ * GLib provides a generic API for computing checksums (or "digests")
+ * for a sequence of arbitrary bytes, using various hashing algorithms
+ * like MD5, SHA-1 and SHA-256. Checksums are commonly used in various
+ * environments and specifications.
+ *
+ * GLib supports incremental checksums using the GChecksum data
+ * structure, by calling g_checksum_update() as long as there's data
+ * available and then using g_checksum_get_string() or
+ * g_checksum_get_digest() to compute the checksum and return it either
+ * as a string in hexadecimal form, or as a raw sequence of bytes. To
+ * compute the checksum for binary blobs and NUL-terminated strings in
+ * one go, use the convenience functions g_compute_checksum_for_data()
+ * and g_compute_checksum_for_string(), respectively.
+ *
+ * Support for checksums has been added in GLib 2.16
+ **/
+
 #define IS_VALID_TYPE(type)     ((type) >= G_CHECKSUM_MD5 && (type) <= G_CHECKSUM_SHA256)
 
+/* The fact that these are lower case characters is part of the ABI */
 static const gchar hex_digits[] = "0123456789abcdef";
 
 #define MD5_DATASIZE    64
@@ -1217,7 +1240,7 @@ g_checksum_update (GChecksum    *checksum,
                    gssize        length)
 {
   g_return_if_fail (checksum != NULL);
-  g_return_if_fail (data != NULL);
+  g_return_if_fail (length == 0 || data != NULL);
 
   if (length < 0)
     length = strlen ((const gchar *) data);
@@ -1255,6 +1278,8 @@ g_checksum_update (GChecksum    *checksum,
  *
  * Once this function has been called the #GChecksum can no longer be
  * updated with g_checksum_update().
+ * 
+ * The hexadecimal characters will be lower case. 
  *
  * Return value: the hexadecimal representation of the checksum. The
  *   returned string is owned by the checksum and should not be modified
@@ -1373,6 +1398,8 @@ g_checksum_get_digest (GChecksum  *checksum,
  * Computes the checksum for a binary @data of @length. This is a
  * convenience wrapper for g_checksum_new(), g_checksum_get_string()
  * and g_checksum_free().
+ * 
+ * The hexadecimal string returned will be in lower case.
  *
  * Return value: the digest of the binary data as a string in hexadecimal.
  *   The returned string should be freed with g_free() when done using it.
@@ -1388,7 +1415,7 @@ g_compute_checksum_for_data (GChecksumType  checksum_type,
   gchar *retval;
 
   g_return_val_if_fail (IS_VALID_TYPE (checksum_type), NULL);
-  g_return_val_if_fail (data != NULL, NULL);
+  g_return_val_if_fail (length == 0 || data != NULL, NULL);
 
   checksum = g_checksum_new (checksum_type);
   if (!checksum)
@@ -1408,6 +1435,8 @@ g_compute_checksum_for_data (GChecksumType  checksum_type,
  * @length: the length of the string, or -1 if the string is null-terminated.
  *
  * Computes the checksum of a string.
+ * 
+ * The hexadecimal string returned will be in lower case.
  *
  * Return value: the checksum as a hexadecimal string. The returned string
  *   should be freed with g_free() when done using it.
@@ -1420,7 +1449,7 @@ g_compute_checksum_for_string (GChecksumType  checksum_type,
                                gssize         length)
 {
   g_return_val_if_fail (IS_VALID_TYPE (checksum_type), NULL);
-  g_return_val_if_fail (str != NULL, NULL);
+  g_return_val_if_fail (length == 0 || str != NULL, NULL);
 
   if (length < 0)
     length = strlen (str);
