@@ -17,10 +17,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
 #include "config.h"
+
 #include "gtestutils.h"
-#include <glib.h>
-#include "galias.h"
+
 #include <sys/types.h>
 #ifdef G_OS_UNIX
 #include <sys/wait.h>
@@ -41,6 +42,14 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif /* HAVE_SYS_SELECT_H */
+
+#include "gmain.h"
+#include "gpattern.h"
+#include "grand.h"
+#include "gstrfuncs.h"
+#include "gtimer.h"
+
+#include "galias.h"
  
 /* Global variable for storing assertion messages; this is the counterpart to
  * glibc's (private) __abort_msg variable, and allows developers and crash
@@ -1255,6 +1264,7 @@ gtest_default_log_handler (const gchar    *log_domain,
                            gpointer        unused_data)
 {
   const gchar *strv[16];
+  gboolean fatal = FALSE;
   gchar *msg;
   guint i = 0;
   if (log_domain)
@@ -1263,7 +1273,10 @@ gtest_default_log_handler (const gchar    *log_domain,
       strv[i++] = "-";
     }
   if (log_level & G_LOG_FLAG_FATAL)
-    strv[i++] = "FATAL-";
+    {
+      strv[i++] = "FATAL-";
+      fatal = TRUE;
+    }
   if (log_level & G_LOG_FLAG_RECURSION)
     strv[i++] = "RECURSIVE-";
   if (log_level & G_LOG_LEVEL_ERROR)
@@ -1282,7 +1295,7 @@ gtest_default_log_handler (const gchar    *log_domain,
   strv[i++] = message;
   strv[i++] = NULL;
   msg = g_strjoinv ("", (gchar**) strv);
-  g_test_log (G_TEST_LOG_ERROR, msg, NULL, 0, NULL);
+  g_test_log (fatal ? G_TEST_LOG_ERROR : G_TEST_LOG_MESSAGE, msg, NULL, 0, NULL);
   g_log_default_handler (log_domain, log_level, message, unused_data);
   g_free (msg);
 }
@@ -1381,7 +1394,7 @@ g_assertion_message_error (const char     *domain,
 			   int             line,
 			   const char     *func,
 			   const char     *expr,
-			   GError         *error,
+			   const GError   *error,
 			   GQuark          error_domain,
 			   int             error_code)
 {
@@ -1416,6 +1429,7 @@ g_assertion_message_error (const char     *domain,
  *
  * Compares @str1 and @str2 like strcmp(). Handles %NULL 
  * gracefully by sorting it before non-%NULL strings.
+ * Comparing two %NULL pointers returns 0.
  *
  * Returns: -1, 0 or 1, if @str1 is <, == or > than @str2.
  *
