@@ -20,6 +20,10 @@
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
+#if defined(G_DISABLE_SINGLE_INCLUDES) && !defined (__GLIB_H_INSIDE__) && !defined (GLIB_COMPILATION)
+#error "Only <glib.h> can be included directly."
+#endif
+
 #ifndef __G_VARIANT_TYPE_H__
 #define __G_VARIANT_TYPE_H__
 
@@ -270,6 +274,7 @@ gboolean                        g_variant_type_is_maybe                 (const G
 gboolean                        g_variant_type_is_array                 (const GVariantType  *type);
 gboolean                        g_variant_type_is_tuple                 (const GVariantType  *type);
 gboolean                        g_variant_type_is_dict_entry            (const GVariantType  *type);
+gboolean                        g_variant_type_is_variant               (const GVariantType  *type);
 
 /* for hash tables */
 guint                           g_variant_type_hash                     (gconstpointer        type);
