@@ -269,6 +269,45 @@ g_byte_array_get_type (void)
     type_id = g_boxed_type_register_static (g_intern_static_string ("GByteArray"),
 					    (GBoxedCopyFunc) g_byte_array_ref,
                                             (GBoxedFreeFunc) g_byte_array_unref);
+
+  return type_id;
+}
+
+GType
+g_variant_type_get_gtype (void)
+{
+  static GType type_id = 0;
+
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GVariantType"),
+                                            (GBoxedCopyFunc) g_variant_type_copy,
+                                            (GBoxedFreeFunc) g_variant_type_free);
+
+  return type_id;
+}
+
+/**
+ * g_variant_get_gtype:
+ *
+ * Since: 2.24
+ * Deprecated: 2.26
+ */
+GType
+g_variant_get_gtype (void)
+{
+  return G_TYPE_VARIANT;
+}
+
+GType
+g_error_get_type (void)
+{
+  static GType type_id = 0;
+
+  if (!type_id)
+    type_id = g_boxed_type_register_static (g_intern_static_string ("GError"),
+                                            (GBoxedCopyFunc) g_error_copy,
+                                            (GBoxedFreeFunc) g_error_free);
+
   return type_id;
 }
 
